#------------------------------------------------------------------
#-   Aug 18 09:37:19 2024
# -  Title: MGBLUP.R
# -  Author: C. Maltecca
#_ 
#-      / \      _-'    | \ | |/ ___| / ___|_   _|/ \|_   _| ____|
#-     /|  \-''- _ /    |  \| | |     \___ \ | | / _ \ | | |  _|
#--' { |          \     | |\  | |___   ___) || |/ ___ \| | | |___ 
#-  /             \     |_| \_|\____| |____/ |_/_/   \_\_| |_____|
#- /      "o.  |o }
#-    |           \;
#-                ',
#-     \_         __\
#-       ''-_    \.//
#-         / '-____'
#-        /
# -      '
# -   -'
# -  Purpose: 
#----------------------------------------------------------------

rm(list=ls())
gc()
require("BGLR")

m_verbose = FALSE
dfig = 2
nIter=5000
burnIn = 100
thin = 50 


setwd("/Users/cmaltec/Desktop")
cat("chain:" , round(nIter-burnIn)/thin,"\n")

load("material/KG.RData")  # load SNPs kernel
load("material/KM.RData")  # load OTU kenrel
load("material/data.RData") # load data (effect plus phenotype)
load("material/M.RData")
load("material/G.RData")




# effect to include
pureff=c("PIG_Gender", "PIG_BirthCG","PIG_BirthLitterID")

# 1. incidence matrix of fixed
f1=formula(paste("~",paste(pureff[1:2],collapse="+")))
print(f1)
fix <- model.matrix(f1, data = pb)
row.names(fix) <- pb$animGLDS

#2. incidence matrix of random
f2=formula(paste0("~",pureff[3]))
rand <- model.matrix(f2, data = pb)[, -1] # remove column of 1's (no mean)
row.names(rand) <- pb$animGLDS

#3. create phenorype
Y=as.double(pb$PIG_LiveAverageDailyGain_W2OF_kg)
#row.names(Y) = pb$animGLDS

################################
## A Simple prediction exercise
################################

samplesize=100
whichNa <- sample(1:length(Y), size = samplesize, replace = FALSE)
yNa <- Y             # make a copy of y. yNa and y are numeric  
yNa[whichNa] <- NA    # set the values of sampled trees to NA


## 1- Predict a phenotype using genomic information 
fmR <- BGLR(y  = yNa,
            ETA = list(MRK = list(X = as.matrix(G), model = "BRR")),
            nIter = 1000, burnIn = 500, thin = 10, verbose = F)


#### *  Calculate model statistics  
df1 <- data.frame(fmR$yHat[whichNa], Y[whichNa])
PredAbi = round( cor(df1[, 2], df1[, 1]), 2)                              
MSE =  round( mean((df1[, 2] - df1[,1])^2), 2)                          
# organize in a table 
table <- cbind(PredAbi, MSE)    
rownames(table) <- c('')
table 


## 2- Predict a phenotype using Microbiome information 
fmR <- BGLR(y  = yNa,
            ETA = list(MRK = list(X = as.matrix(M), model = "BRR")),
            nIter = 1000, burnIn = 500, thin = 10, verbose = F)


#### *  Calculate model statistics  
df1 <- data.frame(fmR$yHat[whichNa], Y[whichNa])
PredAbi = round( cor(df1[, 2], df1[, 1]), 2)                              
MSE =  round( mean((df1[, 2] - df1[,1])^2), 2)                          
# organize in a table 
table <- cbind(PredAbi, MSE)    
rownames(table) <- c('')
table 

## 3- Predict a phenotype using Microbiome and Genomic Information 
fmR <- BGLR(y  = yNa,
            ETA = list(MRK = list(X = as.matrix(G), model = "BRR"),
                       Mic=list(X = as.matrix(M), model = "BRR")),
            nIter = 1000, burnIn = 500, thin = 10, verbose = F)


#### *  Calculate model statistics  
df1 <- data.frame(fmR$yHat[whichNa], Y[whichNa])
PredAbi = round( cor(df1[, 2], df1[, 1]), 2)                              
MSE =  round( mean((df1[, 2] - df1[,1])^2), 2)                          
# organize in a table 
table <- cbind(PredAbi, MSE)    
rownames(table) <- c('')
table 


## 4- Predict a phenotype using Microbiome and Genomic Information + their interactions
library("matrixcalc")

WK<-hadamard.prod(GK, MK)

fmR <- BGLR(y  = yNa,
            ETA = list(MRK = list(K = GK, model = "RKHS",df = dfig),
                       Mic=list(K = MK, model = "RKHS",df = dfig),
                       Inte=list(K = WK, model = "RKHS",df = dfig)),
            nIter = 1000, burnIn = 500, thin = 10, verbose = F)


#### *  Calculate model statistics  
df1 <- data.frame(fmR$yHat[whichNa], Y[whichNa])
PredAbi = round( cor(df1[, 2], df1[, 1]), 2)                              
MSE =  round( mean((df1[, 2] - df1[,1])^2), 2)                          
# organize in a table 
table <- cbind(PredAbi, MSE)    
rownames(table) <- c('')
table 






##############################################
## Predicting Breeding Values accounting for M
##############################################


# set up the three models

GBLUP <- list(  G = list(K = GK, df = dfig, model = "RKHS"),
                FIX = list(X = fix, df = dfig, model = "FIXED"),
                RAND = list(X = rand, df = dfig, model = "BRR")
                )

MBLUP <- list(  M = list(K = MK, df = dfig, model = "RKHS"),
                FIX = list(X = fix, df = dfig, model = "FIXED"),
                RAND = list(X = rand, df = dfig, model = "BRR")
                )


GMBLUP <- list(  G = list(K = GK, df = dfig, model = "RKHS"),
                 M = list(K = MK, df = dfig, model = "RKHS"),
                 FIX = list(X = fix, df = dfig, model = "FIXED"),
                 RAND = list(X = rand, df = dfig, model = "BRR")
                )

CM <- list(  M = list(K = MK, df = dfig, model = "RKHS"))



#####################################################
# Weishaar
# assume all genetic effect mediated by M 
# 1.) estimated effect of microbiota on y (mHat)
# 2.) estimated effect of G on point using mHat as y 
#####################################################

MBLUP=BGLR(y = Y , ETA = MBLUP, nIter = nIter, 
            burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)


mhat=MBLUP$ETA$M$u  
wei=BGLR(y = mhat , ETA = GBLUP, nIter = nIter, 
               burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)

# breeding values
ebv_wei=wei$ETA$G$u 


################################################################################
# Christensen 
# assume genetic effect mediated by M, but also direct genetic effect of y
# 1.) estimated effect of microbiota on y (considering also G) 
# 2.) estimated effect of G on mHat point 1
# 3.) breeding valued sum of G mediated by M plus direct genetic effect point 1.
###############################################################################


GMBLUP=BGLR(y = Y , ETA = GMBLUP, nIter = nIter, 
            burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)

mhat=GMBLUP$ETA$M$u
Chri=BGLR(y = mhat , ETA = GBLUP, nIter = nIter, 
               burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)

ebv_Chri=Chri$ETA$G$u+GBLUP$ETA$G$u

################################################################################
# using GBLUP to calculated residual
# residual new phenotype
# yHat become sum of yHat of GBLUP plus  yHat of model 2
################################################################################

GBLUP=BGLR(y = Y , ETA = GBLUP, nIter = nIter, 
            burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)

res=Y-GBLUP$yHat
CM=BGLR(y = as.matrix(res) , ETA = CM, nIter = nIter, 
               burnIn = burnIn, thin = thin, df = dfig, verbose = m_verbose)

CMyhat=CM$yHat+GBLUP$yHat


